# Lab 4 - API request using Retrofit, ViewModel

## Instructions

- Make regular and well-commented commits as needed
- Push at the end of the session

## Retrofit

Retrofit is a type-safe HTTP client for Android and Java/Kotlin. It simplifies the process of making HTTP requests and parsing the response. Retrofit uses annotations to define the HTTP request. It supports different types of requests: GET, POST, PUT, DELETE, etc. Retrofit also supports synchronous and asynchronous requests.

For more details, read the official documentation: https://square.github.io/retrofit/

## Objective

In this lab, we will see how to use Retrofit to make HTTP requests to the Deezer API. We will create a simple application that displays a list of musical tracks from a given query.
The Deezer API is a music streaming service that provides a RESTful API to access its data. The API documentation can be found here: https://developers.deezer.com/api

## Steps

The objectives of this lab can be broken down in two steps:

- The first step will consist in writing the code to fetch data from the Deezer API and display the request result in the **logcat**.
- Once the first step is functional, the result will be displayed in a structured format within the application using a `RecyclerView`.

### 1. MainActivity

- Create a new project with a main activity `MainActivity` based on the following layout:

<img src="img/layout.png" width=200>

The layout contains:
- a centered title
- a text edit area 
- a button to perform the search
- a `RecyclerView`  to display the results of the search (not visible in the screenshot as it is currently empty)


### 2 - Retrofit

- To add Retrofit, add to the `build.gradle` file:

  ```gradle
  implementation ("com.squareup.retrofit2:retrofit:2.11.0")
  implementation("implementation 'com.squareup.retrofit2:converter-gson:2.11.0")
  ```

- Create a new package `network` and add a new Kotlin file `RetrofitInstance`.
The keyword `object` ensures that the class is a singleton. The method `getInstance` returns an instance of the `Retrofit` class.

  ```kotlin
  object RetrofitInstance {
      private const val BASE_URL = "https://api.deezer.com/"

      fun getInstance() : Retrofit {
          return Retrofit.Builder()
          .baseUrl(BASE_URL)
          .addConverterFactory(GsonConverterFactory.create())
          .build()
      }
  }
  ```
- Create an interface `DeezerApiInterface` that describes the API endpoints. 
In this lab, we will use only the search method:
https://developers.deezer.com/api/search


  ```kotlin
  interface DeezerApiInterface {
      @GET("search")
      suspend fun searchTracks(@Query("q") query:String): DeezerSearchResponse
  }
  ```

Since the HTTP request initiated with this method is asynchronous, it is defined as *suspended* (keyword `suspend`).
This implies that this method can only be invoked within a coroutine.

To understand the format of the response, you can examine the JSON object returned by this URL:

https://api.deezer.com/search?q=miles%20davis

In particular, the `data` field of the response contains a list of tracks.
Track attributes are defined here:
https://developers.deezer.com/api/track

We will retain only a subset of these attributes in the data classes.

### 3 - Data class

In Kotlin, a data class is defined with the keyword `data` and is used to hold data. It automatically generates useful methods like `equals()`, `hashCode()`, `toString()`, and `copy()` based on the properties defined in the class.

- Create a new package `model` and add the following data classes in the file `DeezerSearchResponse.kt`: 

  ```kotlin
  data class DeezerSearchResponse(
      val data : List<Track>
  )

  data class Track(
      val id:String,
      val title: String,
      val duration:String,
      val artist:Artist,
      val album: Album
  )

  data class Album(
      val id:String,
      val title:String,
      val cover:String,
      val tracks:List<Track>
  )

  data class Artist(
      val id:String,
      val name:String,
      val picture:String
  )
  ```

  The data class `DeezerSearchResponse` will hold the request response of the `searchTracks` method in the `DeezerApiInterface`.


### 4 - Interface instanciation

- In the `MainActivity`, create an implementation of the interface  `DeezerApiInterface` using the Retrofit instance:

  ```kotlin
  apiInterface=RetrofitInstance.getInstance().create(DeezerApiInterface::class.java)
  ```

- When the button is clicked, a search is executed on the Deezer API using the text entered in the `EditText` field.
For this, call the `searchTracks(query:String)` of the `DeezerApiInterface`. Since this method runs an asynchronous task, it is marked as `suspend`. Therefore it should be launched within a `CoroutineScope` on a background thread, using the  `Dispatchers.IO` dispatcher to handle this:

  ```kotlin
   CoroutineScope(Dispatchers.IO).launch {
            val response = apiInterface.getSearch(searchText)
            // process the response
            }
   ```

- Process the response by printing in the `logcat` the title of the first album retrieved from the request.

### 5 - Display tracks in the RecyclerView

- Create a layout file `track_item.xml` to hold the view of a result track:
  - the views are organized within a `LinearLayout`, which is placed inside a square-sized`CardView`.
  - the views include: the album cover (as an image), the album title, the track title and the artist name.

  <img src="img/track_item.png" width=200>
 
- In a new package `adapter`, implement  the classes `TrackAdapter`  and `TrackViewHolder` in the file `TrackAdapter.kt`.

- The class `TrackAdapter`:
  - extends `RecyclerView.Adapter<TrackAdapter.TrackViewHolder>`
  - holds a private variable `tracks` of type `List<Track>` (a list of tracks)
  - overrides the three methods `onCreateViewHolder`, `onBindViewHolder`, `getItemCount` (you will use Glide to display the cover image, see Lab3 for all the details)
  - 

- The class `TrackViewHolder`:
  - is an inner class of `TrackAdapter`
  - is responsible for retrieving the views associated with the four fields: album title, song title, artist name, url of the cover. 

- In the `MainActivity`: 
  - set the layout manager as a `GridLayoutManager` with two columns: method `setLayoutManager()` 
  - when the search button is clicked, the resulting tracks are displayed in the `RecyclerView`. To achieve this, set the adapter of the `RecyclerView` to a new `TrackAdapter` with the list of tracks returned by Retrofit.

    **Warning: since this updates the UI, it must be called on the main thread**:

    ```kotlin
     withContext(Dispatchers.Main) {
          recyclerView.adapter = TrackAdapter(response.data)
     }
     ```
 

1. Test your application with multiple queries
2. What happens if a search is initiated with an empty query ?
3. What happens when the screen orientation changes ?

### 6 - ViewModel

To ensure robustness against configuration changes, a good practice is to persist data in a `ViewModel`.
Here, we would like to store the list of tracks returned by the query in a `ViewModel`: since the `ViewModel` survives the configuration change, its data can be retrieved to update the UI.

- In a package `viewmodel`, implement a class `TracksViewModel`
- The class `TracksViewModel`:
  - extends `ViewModel``
  - holds a variable `data`of type `MutableLiveData<DeezerSearchResponse>`

- In the `MainActivity`, instanciate a `TracksViewModel`:
  ```kotlin
  private val viewModel:TracksViewModel by viewModels()
  ```
- On the method `onCreate()`add an observer on the variable `data`:

  ```kotlin
  val observer=Observer<DeezerSearchResponse> {
            response-> recyclerView.adapter=TrackAdapter(response.data)
        }
  viewModel.data.observe(this, observer)
  ```
  This way, each time the data will change, the recyclerview's adapter will be updated.

- Now when a search is performed, the viewmodel will be updated with the data of the response
 

1. Test your application.
2. Test the robustness against configuration change.